//
// File:   main.c
// Author:
//
// Target: PIC18F24K20, PIC18F2520
//
// Description:
//
//  See:  https://www.electro-tech-online.com/threads/reading-a-4x4-switch-matrix.153166/
//
device = 18F2520
clock = 1

include "init_h.bas"
include "init.bas"
include "keypad.bas"
include "lcd.bas"
include "tick.bas"
include "bitdefs.bas"

dim gScanKeypadFlag as byte

//
// Interrupt handlers
//
interrupt ISR_Handler()
    // Handle system tick
    if (INTCON.bits(TMR0IE) = 1) and (INTCON.bits(TMR0IF) = 1) then
        INTCON.bits(TMR0IF) = 0
        if(gScanKeypadFlag = 0) then
            gScanKeypadFlag = 1
        endif
    endif
end interrupt

//
// Display application name and version
//
sub ShowVersion()
    // unsigned char buffer[17] = "\010\011\012\013\014\015\016\017"; // octal byte constants in a string
    const buffer() as byte = ($08, $09, $0A, $0B, $0C, $0D, $0E, 0)

    LCD_SetDDRamAddr(LINE_ONE)
    LCD_WriteConstString(@("Test: LCD+Keypad"))
    // Show what is in the character generator RAM
    LCD_SetDDRamAddr(LINE_TWO)
    LCD_WriteConstString(@buffer)
    LCD_WriteConstString(@(" 18MAR11"))
end sub

//
// Application
//
dim
    Key as byte,
    KeyPressedMessageFlag as byte

main:
    PIC_Init()
    LCD_Init()
    Keypad_Init()
    Tick_Init()

    'INTCONbits.PEIE = 1
    'INTCONbits.GIE  = 1
    enable(ISR_Handler)

    // Display the application name and version information
    ShowVersion()
    KeyPressedMessageFlag = 0

    while (true)
        // Scan the kepad matrix about every 2 milliseconds
        if(gScanKeypadFlag <> 0) then
            gScanKeypadFlag = 0
            Keypad_Scan()
        endif
        // check for and process key presses
        if (Keypad_GetEvent() = eKeyChanged) then
            if (KeyPressedMessageFlag = 0) then
                KeyPressedMessageFlag = 1
                LCD_SetDDRamAddr(LINE_TWO)
                LCD_WriteConstString(@("Key Pressed:    "))
            else
                LCD_SetDDRamAddr(LINE_TWO+13)
                LCD_WriteData(byte(" "))
            endif
            Key = Keypad_GetKey()
            if (Key <> 0) then
                LCD_SetDDRamAddr(LINE_TWO+13)
                LCD_WriteData(Key)
                select (char(Key))
                    case "0"
                    case "1"
                    case "2"
                    case "3"
                    case "4"
                    case "5"
                    case "6"
                    case "7"
                    case "8"
                    case "9"
                    case "E"
                    case "."
                    case "U"
                    case "D"
                    case "L"
                    case "R"
                    else
                        break
                end select
            endif
        endif
    end while

end program
